@@ -31,6 +31,9 @@
 #include "PlatformBackends/ImageGrabber.h"
 #include "ExportMenu.h"
 #include "KSWidget.h"
+#include "SpectacleConfig.h"
+
+class KMoreToolsMenuFactory;
 
 class KSMainWindow : public QDialog
 {
@@ -38,21 +41,38 @@ class KSMainWindow : public QDialog
 
     public:
 
-    explicit KSMainWindow(bool onClickAvailable, QWidget *parent = 0);
-    ~KSMainWindow();
+    explicit KSMainWindow(bool onClickAvailable, QWidget *parent = nullptr);
+    ~KSMainWindow() Q_DECL_OVERRIDE;
+
+    private:
+
+    enum class QuitBehavior {
+        QuitImmediately,
+        QuitExternally
+    };
+    void quit(const QuitBehavior quitBehavior = QuitBehavior::QuitImmediately);
+
+    enum class MessageDuration {
+        AutoHide,
+        Persistent
+    };
+    void showInlineMessage(const QString& message,
+                           const KMessageWidget::MessageType messageType,
+                           const MessageDuration messageDuration = MessageDuration::AutoHide);
 
     private slots:
 
     void captureScreenshot(ImageGrabber::GrabMode mode, int timeout, bool includePointer, bool includeDecorations);
     void showPrintDialog();
+    void openScreenshotsFolder();
     void showPreferencesDialog();
     void showImageSharedFeedback(bool error, const QString &message);
     void sendToClipboard();
     void init();
-    void buildSaveMenu();
+    void setDefaultSaveAction();
     void save();
     void saveAs();
-    void saveAndExit();
+    int windowWidth(const QPixmap &pixmap) const;
 
     public slots:
 
@@ -66,18 +86,25 @@ class KSMainWindow : public QDialog
 
     protected:
 
-    void moveEvent(QMoveEvent *event);
+    void moveEvent(QMoveEvent *event) Q_DECL_OVERRIDE;
 
     private:
 
     KSWidget         *mKSWidget;
     QFrame           *mDivider;
     QDialogButtonBox *mDialogButtonBox;
+    QToolButton      *mConfigureButton;
+    QPushButton      *mToolsButton;
     QPushButton      *mSendToButton;
     QToolButton      *mClipboardButton;
     QToolButton      *mSaveButton;
     QMenu            *mSaveMenu;
+    QAction          *mSaveAsAction;
+    QAction          *mSaveAction;
     KMessageWidget   *mMessageWidget;
+    QMenu            *mToolsMenu;
+    QMenu            *mScreenRecorderToolsMenu;
+    QScopedPointer<KMoreToolsMenuFactory> mScreenrecorderToolsMenuFactory;
     ExportMenu       *mExportMenu;
     bool              mOnClickAvailable;
 };
